SystemOrganization addCategory: #'Seaside-Adaptors-Swazoo'!

WAStartableServerAdaptor subclass: #WASwazooAdaptor
	instanceVariableNames: 'site'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Adaptors-Swazoo'!

!WASwazooAdaptor class methodsFor: 'testing' stamp: 'lr 2/15/2009 18:01'!
isAbstract
	^ self name = #WABaseServerAdaptor! !

!WASwazooAdaptor class methodsFor: 'actions' stamp: 'jf 2/15/2009 23:03'!
resetAll
	"removes all Seaside sites, run this if you run into trouble creating servers"
	| server toRemove |
	server := SwazooServer singleton.
	toRemove := server allSites select: [ :each |
		each isKindOf: WAPluggableSite ].
	toRemove do: [ :each |
		server removeSite: each ]! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:41'!
basicStart
	site port: self port.
	site start! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:44'!
basicStop
	site stop! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:53'!
convertCookiesFrom: aWAResonse to: aSwazooResponse
	| cookies setCookieField setCookie2Field |
	cookies := aWAResonse cookies.
	cookies isEmpty ifTrue: [ ^self ].
	
	setCookieField := WorkingHTTPSetCookieField new.
	setCookie2Field := WorkingHTTPSetCookie2Field new.

	cookies do: [ :each |
		setCookieField addCookie: each oldNetscapeString.
		setCookie2Field addCookie: each rfc2965String ].
	
	aSwazooResponse headers addField: setCookieField.
	aSwazooResponse headers addField: setCookie2Field! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:49'!
convertHeader: aSwazooRequest
	| swazooFields fields |
	swazooFields := aSwazooRequest headers fields.
	fields := Dictionary new: swazooFields size.
	swazooFields keysAndValuesDo: [ :key :field |
		| value |
		value := String streamContents: [ :stream |
			field valuesAsStringOn: stream ].
		fields at: key asLowercase put: value ].
	^fields! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:42'!
cookiesFrom: aSwazooRequest
	^WARequestCookie fromString: aSwazooRequest cookie! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'lr 2/15/2009 19:19'!
fieldsFor: aNativeRequest
	| merger |
	merger := WADictionaryMerger new.
	self getFieldsOf: aNativeRequest into: merger.
	(self isPost: aNativeRequest) 
		ifTrue: [ self postFieldsOf: aNativeRequest into: merger ].
	^ merger dictionary! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'lr 2/15/2009 20:01'!
getFieldsOf: aSwazooRequest into: aDictionaryMerger
	aSwazooRequest queries do: [ :each |
		aDictionaryMerger at: each key put: (self codec url decode: each value) ]! !

!WASwazooAdaptor methodsFor: 'initialization' stamp: 'jf 2/15/2009 23:03'!
initialize
	super initialize.
	site := WAPluggableSite
		named: (WAKeyGenerator current keyOfLength: 32)
		on: self.
	site ip: '*'.
	site host: '*'.
	SwazooServer singleton addSite: site! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'lr 2/15/2009 18:00'!
isPost: aNativeRequest
	^(self methodNameOf: aNativeRequest) = 'POST'! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'lr 2/15/2009 18:02'!
isRunning
	^ site isServing! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'lr 2/15/2009 17:47'!
isStarting
	^ false! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'lr 2/15/2009 18:02'!
isStopped
	^ site isServing not! !

!WASwazooAdaptor methodsFor: 'testing' stamp: 'lr 2/15/2009 18:02'!
isStopping
	^ false! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/27/2008 07:53'!
methodNameOf: aNativeRequest
	^aNativeRequest methodName! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'lr 2/15/2009 20:01'!
postFieldsOf: aNativeRequest into: aDictionaryMerger
	"break encapsulation"
	aNativeRequest ensureFullRead.
	aNativeRequest postData keysAndValuesDo: [ :key :value |
		| previousData data |
		data := value filename isEmptyOrNil
			ifTrue: [ self codec url decode: value value ]
			ifFalse: [
				WAFile new
					fileName: (self codec url decode: value filename);
					contentType: value contentType seasideMimeType;
					contents: value value;
					yourself ].
		aDictionaryMerger at: key put: data ]! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'pmm 11/6/2008 14:57'!
remoteAddressOf: aSwazooRequest
	"converts a Squeak address (ByteArray) to an IPv4 String"
	| remoteAddress |
	remoteAddress := aSwazooRequest peer.
	remoteAddress isString ifTrue: [
		"so apparently someone has a new VM with IPv6 support"
		^remoteAddress ].
	
	^String streamContents: [ :stream |
		remoteAddress
			do: [ :each | stream print: each ]
			separatedBy: [ stream nextPut: $. ] ]! !

!WASwazooAdaptor methodsFor: 'converting' stamp: 'lr 2/15/2009 19:55'!
requestFor: aNativeRequest
	^ WARequest
		method: (self methodNameOf: aNativeRequest)
		url: (self urlFor: aNativeRequest)
		headers: (self convertHeader: aNativeRequest)
		fields: (self fieldsFor: aNativeRequest)
		cookies: (self cookiesFrom: aNativeRequest)
		remoteAddress: (self remoteAddressOf: aNativeRequest)
		nativeRequest: aNativeRequest! !

!WASwazooAdaptor methodsFor: 'converting' stamp: 'lr 2/15/2009 20:01'!
responseFrom: aRequestContext
	| swazooResponse |
	swazooResponse := HTTPResponse new.
	aRequestContext response headers keysAndValuesDo: [ :key :value |
		key = 'Content-Type'
			ifTrue: [ swazooResponse contentType: value seasideString ]
			ifFalse: [
				| field fieldClass |
				fieldClass := HeaderField classForFieldName: key.
				field := fieldClass newForFieldName: key withValueFrom: value seasideString.
				swazooResponse headers addField: field ] ].
	self convertCookiesFrom: aRequestContext response to: swazooResponse.
	swazooResponse code: aRequestContext response status asNumber.
	swazooResponse entity: aRequestContext response contents.
	^ swazooResponse! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/8/2008 12:46'!
shutDown
	SwazooServer stop! !

!WASwazooAdaptor methodsFor: 'private-controlling' stamp: 'pmm 11/15/2008 10:45'!
startUp
	SwazooServer start! !

!WASwazooAdaptor methodsFor: 'private' stamp: 'lr 2/15/2009 19:17'!
urlFor: aSwazooRequest
	^ ((WAUrl absolute: aSwazooRequest uri printString)
		decodedWith: self codec) withoutParameters! !

!WAPackage class methodsFor: '*seaside-adaptors-swazoo' stamp: 'lr 12/18/2008 20:25'!
seasideAdaptorsSwazoo
	^ self new
		name: 'Seaside-Adaptors-Swazoo';
		description: 'Makes Seaside run on the Swazoo Web server (LGPL).';
		addDependency: 'Seaside-Squeak-Core';
		addDependency: 'Seaside-Swazoo';
		yourself! !
